'use client';

import { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { useToast } from '@/components/ui/use-toast';
import { Toaster } from '@/components/ui/toaster';
import { DashboardLayout } from '@/components/layouts/dashboard-layout';
import  ApiKeyAnalytics from '@/components/api-keys/analytics';
import { type ApiKey } from '@/types/api';
import { CreateKeyDialog } from './components/create-key-dialog';
import { KeySearch } from './components/key-search';
import { KeyList } from './components/key-list';
import { KeyPagination } from './components/key-pagination';
import { fetchApiKeys, createApiKey, deleteApiKey, revokeApiKey, reactivateApiKey, updateApiKeyTags } from './api';
import { useCallback } from 'react';

const ITEMS_PER_PAGE = 10;

export default function ApiKeysPage() {
  const { toast } = useToast();
  const navigate = useNavigate();

  const [isLoading, setIsLoading] = useState(true);
  const [apiKeys, setApiKeys] = useState<ApiKey[]>([]);
  const [totalKeys, setTotalKeys] = useState(0);
  const [currentPage, setCurrentPage] = useState(1);
  const [searchQuery, setSearchQuery] = useState('');
  const [isCreating, setIsCreating] = useState(false);
  const [newApiKey, setNewApiKey] = useState<string>('');
  const [allowCloseOnOutsideClick, setAllowCloseOnOutsideClick] = useState(true);


  const loadApiKeys = useCallback(async (page: number) => {
    const data = await fetchApiKeys(toast, navigate, page, ITEMS_PER_PAGE);

    if(data) {
      setApiKeys(data.keys);
      setTotalKeys(Number(data.total));
      setCurrentPage(page);
    }

    setIsLoading(false);
  }, [toast, navigate]);

  useEffect(() => {
    loadApiKeys(1);
  }, [loadApiKeys]);


  const handleCreateApiKey = async (name: string, expiresAt: string, tags: string[]) => {
      const response = await createApiKey(toast, navigate, name, expiresAt, tags);

      if (!response) {
        setIsCreating(false);
        setNewApiKey('');
        setAllowCloseOnOutsideClick(true);
        return
      }

      setNewApiKey(response.api_key || '');
      await loadApiKeys(currentPage);
      setAllowCloseOnOutsideClick(false);
    };

  const handleDeleteApiKey = async (keyId: string) => {
    const response = await deleteApiKey(toast, navigate, keyId);

    if(response) {
      toast({
        title: "Success",
        description: "The API key has been deleted successfully.",
      });
  
      loadApiKeys(currentPage);  
    }
  };

  const handleRevokeApiKey = async (keyId: string) => {
    const response = await revokeApiKey(toast, navigate, keyId);

    if(response) {
      toast({
        title: "Success",
        description: "The API key has been revoked successfully.",
      });
      loadApiKeys(currentPage);
    }
  };

  const handleReactivateApiKey = async (keyId: string) => {
    const response = await reactivateApiKey(toast, navigate, keyId);

    if(response) {
      toast({
        title: "Success",
        description: "The API key has been reactivated successfully.",
      });
      loadApiKeys(currentPage);
    }
  };

  const handleUpdateApiKeyTags = async (keyId: string, tags: string[]) => {
   const response = await updateApiKeyTags(toast, navigate, keyId, tags);

   if(response) {
     loadApiKeys(currentPage);
   }
  };

  const copyToClipboard = async (text: string) => {
    try {
      await navigator.clipboard.writeText(text);
      toast({
        title: "Copied!",
        description: "API key copied to clipboard",
      });
    } catch {
      toast({
        variant: "destructive",
        title: "Failed to copy",
        description: "Please try copying manually",
      });
    }
  };

  const handleDialogOpenChange = (open: boolean) => {
    setIsCreating(open);
    if (!open) {
      setNewApiKey('');
      setAllowCloseOnOutsideClick(true);
    }
  };

  if (isLoading) {
    return (
      <div className="min-h-screen flex items-center justify-center">
        <div className="animate-spin rounded-full h-8 w-8 border-t-2 border-b-2 border-gray-900" />
      </div>
    );
  }

  return (
    <DashboardLayout>
      <div className="space-y-6">
        <div className="flex items-center justify-between">
          <h2 className="text-2xl font-semibold text-gray-900">API Keys</h2>
          <CreateKeyDialog
            isOpen={isCreating}
            onOpenChange={handleDialogOpenChange}
            onCreateKey={handleCreateApiKey}
            onCopyKey={copyToClipboard}
            newApiKey={newApiKey}
            allowCloseOnOutsideClick={allowCloseOnOutsideClick}
          />
        </div>

        <ApiKeyAnalytics keys={apiKeys} totalKeys={totalKeys} />

        <div className="space-y-4">
          <KeySearch value={searchQuery} onChange={setSearchQuery} />
          <KeyList
            keys={apiKeys}
            searchQuery={searchQuery}
            onDelete={handleDeleteApiKey}
            onRevoke={handleRevokeApiKey}
            onReactivate={handleReactivateApiKey}
            onUpdateTags={handleUpdateApiKeyTags}
          />
          {!searchQuery && (
            <KeyPagination
              currentPage={currentPage}
              totalItems={totalKeys}
              itemsPerPage={ITEMS_PER_PAGE}
              onPageChange={loadApiKeys}
            />
          )}
        </div>
      </div>
      <Toaster />
    </DashboardLayout>
  );
}
